package cz.zweistein.df.soundsense.output.sound;

import java.util.logging.Logger;
import java.util.regex.Pattern;

import cz.zweistein.df.soundsense.config.Sound;
import cz.zweistein.df.soundsense.config.SoundsXML;
import cz.zweistein.df.soundsense.output.Procesor;
import cz.zweistein.df.soundsense.output.sound.player.PlayerManager;
import cz.zweistein.df.soundsense.output.sound.tts.TextToSpeechManager;
import cz.zweistein.df.soundsense.util.LoggerSource;

public class SoundProcesor extends Procesor {
	private static Logger logger = LoggerSource.logger;
	
	private SoundsXML soundsXML;
	private PlayerManager player;
	private TextToSpeechManager tts;
	
	public SoundProcesor(SoundsXML soundsXML) {
		this.soundsXML = soundsXML;
		this.player = new PlayerManager();
		this.tts = new TextToSpeechManager();
	}
	
	public void setGlobalVolume(float globalVolume) {
		this.player.setGlobalVolume(globalVolume);
	}

	@Override
	public void processLine(String nextLine) {
		logger.info(nextLine);
		
		boolean matched = false;
		int matches = 0;
		for (Sound sound: soundsXML.getSounds()) {
			
			if (Pattern.matches(sound.getLogPattern(), nextLine)) {
				matches++;
				matched = true;
				logger.fine("Message '"+nextLine+"' matched rule '"+sound.getLogPattern()+"' from '"+sound.getParentFile()+"'.");
				if (sound.getSpeech() == false) {
					player.playSound(sound);
				} else {
					tts.play(nextLine);
				}
				if (sound.getHaltOnMatch() == true) {
					logger.finest("Ending matching prematurely as expected.");
					break;
				} else {
					logger.fine("Continuing for next rule match.");
				}
			}
		}
		if (matched == false) {
			logger.fine("Message '"+nextLine+"' did not match any rule.");
		} else {
			logger.finest("Message '"+nextLine+"' matched " + matches + " rules.");
		}
	}

}
