package cz.zweistein.df.soundsense.output.sound.player;

import java.util.logging.Logger;

import cz.zweistein.df.soundsense.config.Sound;
import cz.zweistein.df.soundsense.util.LoggerSource;

public class ChannelThread implements Runnable {
	private static Logger logger = LoggerSource.logger;
	
	private SPIPlayer musicPlayer;
	private Sound loopMusic;
	private String currentMusic;
	private boolean temporaryLoopDisable;
	private float defaultGain = 0;
	
	private String channelName;

	public void setDefaultGain(float defaultGain) {
		logger.finest("Setting "+defaultGain+" gain for "+channelName);
		this.defaultGain = defaultGain;
		if (musicPlayer != null) {
			musicPlayer.setMasterGain(defaultGain);
		}
	}

	public ChannelThread(String channelName) {
		this.channelName = channelName;
	}

	public void setTemporaryLoopDisable(boolean temporaryLoopDisable) {
		this.temporaryLoopDisable = temporaryLoopDisable;
	}

	public void setLoopMusic(Sound sound) {
		this.loopMusic = sound;
	}

	public void setCurrentMusic(String sound) {
		this.currentMusic = sound;
	}

	public SPIPlayer getMusicPlayer() {
		return this.musicPlayer;
	}

	@Override
	public void run() {
		
		while(true) {
			
			try {
				
				if (currentMusic != null) {
			
					logger.finest("Channel playing "+currentMusic);
		
					musicPlayer = new SPIPlayer();
					
					logger.fine(channelName+": "+currentMusic);
					musicPlayer.setMasterGain(this.defaultGain);
					musicPlayer.play(currentMusic);
					
					if (!temporaryLoopDisable) {
						if (loopMusic != null) {
							currentMusic = loopMusic.getRandomSoundFile().getFileName();
							logger.fine("Channel looping "+channelName+": "+currentMusic);
						} else {
							currentMusic = null;
						}
					} else {
						temporaryLoopDisable = false;
					}
				
				} else {
					Thread.sleep(100);
				}
			
			} catch (Exception e) {
				logger.severe("Exception :"+e+": "+e.toString());
				try {
					Thread.sleep(100);
				} catch (InterruptedException e1) {
				}
			}
			
			
		} 
		
	}

}
