package cz.zweistein.df.soundsense.output.sound.player;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import cz.zweistein.df.soundsense.config.Loop;
import cz.zweistein.df.soundsense.config.Sound;
import cz.zweistein.df.soundsense.util.LoggerSource;


public class PlayerManager {
	private static Logger logger = LoggerSource.logger;
	
	public int concurentSounds = 0;
	private float globalChanngelGain = 0;
	private float globalVolume;

	private Map<String, ChannelThread> channels;
	private Set<MP3Thread> sfxThreads;
	
	private VolumeAdjuster volumeAdjuster;
	
	public PlayerManager() {
		this.channels = new HashMap<String, ChannelThread>();
		this.sfxThreads = new HashSet<MP3Thread>();
		
		this.volumeAdjuster = new VolumeAdjuster();
		this.volumeAdjuster.setManager(this);
		new Thread(this.volumeAdjuster).start();
	}

	public void playSound(Sound sound) {
		
		if (sound.getTimeout() != null) {
			if ((sound.getLastPlayed()) < System.currentTimeMillis() - sound.getTimeout()) {
				sound.setLastPlayed(System.currentTimeMillis());
			} else {
				logger.fine("Sound " + sound.getLogPattern() + " is still timeouted, will be ready in " +
						((sound.getLastPlayed() + sound.getTimeout()) - System.currentTimeMillis()) + ".");
				return;
			}
		}
		
		if (sound.getChannel() == null) {
			if (sound.getConcurency() == null || (sound.getConcurency() != null && concurentSounds < sound.getConcurency())) {
				MP3Thread mp3Thread = new MP3Thread(this, sound, this.globalVolume);
				new Thread(mp3Thread).start();
			} else {
				logger.fine("Dropped sound for "+sound.getLogPattern()+" its concurency is "+sound.getConcurency()+" and there are "+concurentSounds+" sounds playing.");
			}
			
		} else  {
			
			ChannelThread channelThread = this.getChannel(sound.getChannel());
			
			if (sound.getLoop() == Loop.START_LOOPING) {
				channelThread.setLoopMusic(sound);
			} else if (sound.getLoop() == Loop.STOP_LOOPING) {
				channelThread.setLoopMusic(null);
				logger.fine("Stopped looping "+sound.getChannel()+".");
			}
			
			if (sound.hasNoSoundFiles()) {
				channelThread.setCurrentMusic(null);
				logger.finest("Stopped playin "+sound.getChannel()+".");
			} else {
				channelThread.setCurrentMusic(sound.getRandomSoundFile().getFileName());
			}

			if (channelThread.getMusicPlayer() != null) {
				channelThread.getMusicPlayer().stopPlayback();
			}
	
			channelThread.setTemporaryLoopDisable(true);
				
		}
		
	}

	private ChannelThread getChannel(String channel) {
		ChannelThread channelThread = this.channels.get(channel);
		if (channelThread == null) {
			channelThread = new ChannelThread(channel);
			channelThread.setDefaultGain(this.globalChanngelGain+this.globalVolume);
			this.channels.put(channel, channelThread);
			new Thread(channelThread).start();
		}
		return channelThread;
	}
	
	public synchronized void setGainForAllChannels(float gain) {
		this.globalChanngelGain = gain;
		for(Map.Entry<String, ChannelThread> entry: this.channels.entrySet()) {
			entry.getValue().setDefaultGain(gain+this.globalVolume);
		}
	}
	
	public synchronized void setVolumeForAllSfx(float volume) {
		for (MP3Thread sfx: this.sfxThreads) {
			sfx.setVolume(volume);
		}
	}

	public synchronized void mp3ThreadPlayingStartCallback(MP3Thread mp3Thread) {
		this.sfxThreads.add(mp3Thread);
		concurentSounds++;
		this.volumeAdjuster.setTargetGain(-10);
	}
	
	public synchronized void mp3ThreadPlayingEndCallback(MP3Thread mp3Thread) {
		this.sfxThreads.remove(mp3Thread);
		concurentSounds--;
		if (this.concurentSounds == 0) {
			this.volumeAdjuster.setTargetGain(0);
		}
	}

	public void setGlobalVolume(float globalVolume) {
		this.globalVolume = globalVolume;
		this.setGainForAllChannels(this.globalChanngelGain);
		this.setVolumeForAllSfx(globalVolume);
	}
	
}
