package cz.zweistein.df.soundsense.output.sound.player;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

import cz.zweistein.df.soundsense.util.LoggerSource;

public class SPIPlayer {
	private static Logger logger = LoggerSource.logger;
	
	private boolean stop = false;
	private float masterGain = 0;
	
	/**
	 * @param masterGain - decibel value of volume amplification.
	 */
	public void setMasterGain(float masterGain) {
		this.masterGain = masterGain;
	}

	public void stopPlayback() {
		this.stop = true;
	}
	
	public void play(String fileName) {
		try {
			logger.finest("Opening '"+fileName+"'");
			File file = new File(fileName);
			AudioInputStream in = AudioSystem.getAudioInputStream(file);
			AudioInputStream din = null;
			AudioFormat baseFormat = in.getFormat();
			AudioFormat decodedFormat = new AudioFormat(
					AudioFormat.Encoding.PCM_SIGNED,
					baseFormat.getSampleRate(), 16, baseFormat.getChannels(),
					baseFormat.getChannels() * 2, baseFormat.getSampleRate(),
					false);
			din = AudioSystem.getAudioInputStream(decodedFormat, in);
			// Play now.
			rawplay(decodedFormat, din);
			in.close();
		} catch (Exception e) {
			logger.severe("Exception with "+fileName+": "+e+": "+e.getMessage());
		}
	}

	private void rawplay(AudioFormat targetFormat, AudioInputStream din)
			throws IOException, LineUnavailableException {
		byte[] data = new byte[4096];
		SourceDataLine line = getLine(targetFormat);
		
		
		
		if (line != null) {
			// Start
			line.start();
			int nBytesRead = 0, nBytesWritten = 0;
			while (nBytesRead != -1) {
				if (this.stop == true) {
					logger.finest("Stopping playback.");
					this.stop = false;
					break;
				}
				if (this.masterGain != 0) {
					FloatControl masterGainControl = (FloatControl) line.getControl(FloatControl.Type.MASTER_GAIN);
					
					if (this.masterGain < masterGainControl.getMinimum() ) {
						this.masterGain = masterGainControl.getMinimum();
					} else if (this.masterGain > masterGainControl.getMaximum() ) {
						this.masterGain = masterGainControl.getMaximum();
					}
					
					masterGainControl.setValue(this.masterGain);
					this.masterGain = 0;
				}
				nBytesRead = din.read(data, 0, data.length);
				if (nBytesRead != -1) {
					nBytesWritten = line.write(data, 0, nBytesRead);
				}
			}
			// Stop
			line.drain();
			line.stop();
			line.close();
			din.close();
		}
	}

	private SourceDataLine getLine(AudioFormat audioFormat)
			throws LineUnavailableException {
		SourceDataLine res = null;
		DataLine.Info info = new DataLine.Info(SourceDataLine.class,
				audioFormat);
		res = (SourceDataLine) AudioSystem.getLine(info);
		res.open(audioFormat);
		return res;
	}

}
